
Tile = {}

Tile.__index = Tile

function Tile.new(x,y,type,i)
  
  local self = {}
  
  self.x = x
  self.y = y
  self.w = 50
  self.h = 50
  self.type = type
  self.clicked = false
  self.mouseover = false
  self.f = 0
  self.adjacent = 0
  self.i = i
  self.spread = 0
  self.marked = false
  
  return setmetatable(self,Tile)
end

local firstMove = false

function Tile.update(self,dt)
  self.adjacent = 0
  self.mouseover = ((love.mouse.getX() > self.x) and (love.mouse.getX() < self.x + self.w) and (love.mouse.getY() > self.y) and (love.mouse.getY() < self.y + self.h))
  
  
  if(self.mouseover and love.mouse.isDown(1) and not self.clicked and not self.marked and firstMove) then
    self.clicked = true
    self:ejectParticles(5 * (self.adjacent+1))
    if(self.type == 1) then
      explosion:stop()
      explosion:play()
      revealBombs()
      alive = false
      lose:play()
    else
      self:calculateScore()
      click:stop()
      click:play()
      click2:stop()
      click2:play()
    end
  end
  
  
  if(self.mouseover) then
    self.f = 0
  else
    self.f = 15
  end
  
  if(self.spread > 0) then
    self:openAdjacentTiles()
  end
  
  if(self.type == 0) then
    self:calculateAdjacent()
  end

end

function Tile.mousepressed(self,x,y,button)
  if(self.mouseover and button == 1 and not self.clicked and not self.marked) then
    self.clicked = true
    self:ejectParticles(5 * (self.adjacent+1))
    if(self.type == 0 and not firstMove) then
      explosion2:play()
      first:play()
      self.spread = 50
      self:openAdjacentTiles()
      firstMove = true
    end
    
    if(self.type == 1 and not firstMove) then
      self.type = 0
      explosion2:play()
      first:play()
      self.spread = 50
      self:openAdjacentTiles()
      
      table.remove(grid.bombs,self.i)
      repeat
        x = math.random(1,99)
      until (not contains(grid.bombs,x))
      grid.tiles[x].type = 1
      
      table.insert(grid.bombs,#grid.bombs+1,grid.tiles[x])
      
      firstMove = true
    end
    
    if(self.type == 1) then
      self:ejectParticles(20)
      explosion:stop()
      explosion:play()
      revealBombs()
      alive = false
      lose:play()
    else
      self:calculateScore()
      click:play()
      click2:play()
    end
  end
  
  if(self.mouseover and button == 2 and not self.clicked) then
    if(self.marked) then
      bombs = bombs + 1
      self.marked = false
      flag:play()
    elseif(not self.marked and bombs > 0) then
      bombs = bombs - 1
      self.marked = true
      flag:play()
    end
  end
end

function revealBombs()
  for i = 1, 99 do
    if(grid.tiles[i].type == 1) then
      grid.tiles[i].clicked = true
    end
  end
end

function getBombCount()
  local x = 0
  for i = 1, 99 do
    if(grid.tiles[i].type == 1) then
      x = x + 1
    end
  end
  return x
end

function Tile.calculateScore(self)
  
  local cal = mapColor(self.adjacent,0,8,20,1)
  
  score = score + cal
end

function Tile.ejectParticles(self,amount)
  for i = 1,amount do
    table.insert(grid.particles,#grid.particles,Particles.new(self.x+25+(math.random(-5,5)),self.y+25+(math.random(-5,5))))
  end
end

function Tile.openAdjacentTiles(self)
  
  local s = 5 + level
  
  if((self.x ~= 400) and grid.tiles[self.i + 11].type == 0) then
     grid.tiles[self.i + 11].clicked = true
     grid.tiles[self.i + 11].spread = self.spread - s
     grid.tiles[self.i+11]:ejectParticles(1)
     score = score + 1
    
  end
  if((self.x ~= 0) and grid.tiles[self.i - 11].type == 0) then
     grid.tiles[self.i - 11].clicked = true
     grid.tiles[self.i - 11].spread = self.spread - s
     grid.tiles[self.i-11]:ejectParticles(1)
     score = score + 1
  end
  
  if((self.y ~= 500) and grid.tiles[self.i + 1].type == 0) then
     grid.tiles[self.i + 1].clicked = true
     grid.tiles[self.i + 1].spread = self.spread - s
      grid.tiles[self.i+1]:ejectParticles(1)
     score = score + 1
  end
  if((self.y ~= 0) and grid.tiles[self.i - 1].type == 0) then
     grid.tiles[self.i - 1].clicked = true
     grid.tiles[self.i - 1].spread = self.spread - s
     grid.tiles[self.i-1]:ejectParticles(1)
     score = score + 1
  end
  
  self.spread = 0
  
end



function Tile.calculateAdjacent(self)
  if((self.x ~= 400) and grid.tiles[self.i + 11].type == 1) then
    self.adjacent = self.adjacent + 1
  end
  if((self.x ~= 0) and grid.tiles[self.i - 11].type == 1) then
    self.adjacent = self.adjacent + 1
  end
  
  if((self.y ~= 500) and grid.tiles[self.i + 1].type == 1) then
    self.adjacent = self.adjacent + 1
  end
  if((self.y ~= 0) and grid.tiles[self.i - 1].type == 1) then
    self.adjacent = self.adjacent + 1
  end
  
  if((self.y > 0 and self.x > 0) and grid.tiles[self.i - 12].type == 1) then -- top left
    self.adjacent = self.adjacent + 1
  end
  if((self.y > 0 and self.x < 400) and grid.tiles[self.i + 10].type == 1) then -- top right
    self.adjacent = self.adjacent + 1
  end
  
  
  if((self.x < 400 and self.y < 500) and grid.tiles[self.i + 12].type == 1) then -- bottom right
    self.adjacent = self.adjacent + 1
  end
  if( (self.x > 0 and self.y < 500) and grid.tiles[self.i - 10].type == 1) then -- bottom left
    self.adjacent = self.adjacent + 1
  end
  
  
end


function mapColor(x,in_min,in_max,out_max,out_min)
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
end

function Tile.draw(self)
  love.graphics.setLineWidth(5)
  if(not self.clicked) then
    love.graphics.setColor(150 + self.f,150 + self.f,150 + self.f)
    love.graphics.rectangle("fill",self.x,self.y,self.w,self.h)
    love.graphics.setColor(200 + self.f,200 + self.f,200 + self.f)
    love.graphics.rectangle("line",self.x+3,self.y+3,self.w,self.h)
    love.graphics.setColor(100 + self.f,100 + self.f,100 + self.f)
    love.graphics.rectangle("line",self.x-2,self.y-2,self.w,self.h)
    
    
    if(self.marked) then
      love.graphics.setColor(255,0,255)
      love.graphics.print("F",self.x + 10,self.y+10)
    end
    
  else
    love.graphics.setColor(100,100,100)
    love.graphics.rectangle("fill",self.x,self.y,self.w,self.h)
    love.graphics.setColor(95,95,95)
    love.graphics.rectangle("line",self.x,self.y,self.w,self.h)
    
    if(self.type == 0 and self.adjacent > 0) then
      love.graphics.setColor(mapColor(self.adjacent,0,8,255,0),50,70)
      love.graphics.print(self.adjacent,self.x + 17,self.y+12)
    elseif(self.type == 1) then
      love.graphics.setColor(0,0,0)
      love.graphics.print("B",self.x + 17,self.y+12)
    end
    
    
  end
  
end


function Tile.reset(self)
  self.type = 0
  self.clicked = false
  self.mouseover = false
  self.adjacent = 0
  self.spread = 0
  self.marked = false
  firstMove = false
end